<?php

namespace Drupal\blurhash_image_style_metadata\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

final class BlurhashImageStyleMetadataSettingsForm extends ConfigFormBase {

  protected function getEditableConfigNames(): array {
    return ['blurhash_image_style_metadata.settings'];
  }

  public function getFormId() {
    return 'blurhash_image_style_metadata_settings_form';
  }

  public function buildForm(array $form, FormStateInterface $form_state) {
    $saved_config = $this->config('blurhash_image_style_metadata.settings');

    $form['scale_down'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Scale down images to generate blurhash'),
      '#default_value' => $saved_config->get('scale_down'),
      '#description' => $this->t('Creating blurhash with very large images can consume too many resources.<br>If enabled, images will be scaled down before generating the blurhash.'),
    ];


    $form['max_width'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Max width'),
      '#default_value' => $saved_config->get('max_width'),
      '#description' => $this->t('The maximum width of the image to generate the blurhash.'),
      '#states' => [
        'visible' => [
          ':input[name="scale_down"]' => ['checked' => TRUE],
        ],
      ],
    ];
    $form['max_height'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Max height'),
      '#default_value' => $saved_config->get('max_height'),
      '#description' => $this->t('The maximum height of the image to generate the blurhash.'),
      '#states' => [
        'visible' => [
          ':input[name="scale_down"]' => ['checked' => TRUE],
        ],
      ],
    ];

    return parent::buildForm($form, $form_state);
  }

  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
    $scale_down = $form_state->getValue('scale_down');
    $max_width = $form_state->getValue('max_width');
    $max_height = $form_state->getValue('max_height');
    if ($scale_down && !$max_width && !$max_height) {
      $form_state->setErrorByName('max_width', $this->t('If scale down is enabled, you must fill in either the maximum width, or the maximum height, or both.'));
      $form_state->setErrorByName('max_height', $this->t('If scale down is enabled, you must fill in either the maximum width, or the maximum height, or both.'));
    }

    if ($scale_down && $max_width) {
      if (!is_numeric($max_width)) {
        $form_state->setErrorByName('max_width', $this->t('Max width must be an integer.'));
      }
      if ($max_width < 10) {
        $form_state->setErrorByName('max_width', $this->t('Max width cannot be less than 10 pixels.'));
      }
    }
    if ($scale_down && $max_height) {
      if (!is_numeric($max_height)) {
        $form_state->setErrorByName('max_height', $this->t('Max height must be an integer.'));
      }
      if ($max_height < 10) {
        $form_state->setErrorByName('max_height', $this->t('Max height cannot be less than 10 pixels.'));
      }
    }

  }

  public function submitForm(array &$form, FormStateInterface $form_state) {
    $scale_down = $form_state->getValue('scale_down');
    if ($scale_down) {
      $this->config('blurhash_image_style_metadata.settings')
        ->set('scale_down', $form_state->getValue('scale_down'))
        ->set('max_width', $form_state->getValue('max_width'))
        ->set('max_height', $form_state->getValue('max_height'))
        ->save();
    }
    else {
      $this->config('blurhash_image_style_metadata.settings')
        ->set('scale_down', $form_state->getValue('scale_down'))
        ->set('max_width', NULL)
        ->set('max_height', NULL)
        ->save();
    }
    parent::submitForm($form, $form_state);
  }

}
